% Generate graphs: 
% Heterogeneous coefficients and probability of high-lambda group vs bank characteristics
% (for selected samples: v_smpl0 = RRE and CC samples starting at 2007Q2)

%% Setup style
graph_size = [6 6];
lgd = [];
SFont = 16;

bankvar_names = {'logassets','loanfrac','capital2assets','loan2assets','alll2loan', ...
    'diversification','roa','oca'}; % tags in file names
bankvar_print = {'Log Assets','Loan Fraction','Capital-Asset','Loan-Asset','ALLL-Loan', ...
    'Diversification','Ret. on Assets','OCA'}; % labels in output graphs
n_bankvar = length(bankvar_names);

n_group = 2;
color_group = [0 0 1; 1 0 0];

i_est = 1; % Flexible CRE, heterosk.
is_load_est = 1; % load estimation results

%% Loop over samples
for i_smpl0 = 1:n_smpl0
    %% Prepare: sample i
    sub_smpl_i; % subroutine: prepare sample i
    
    %% y variable = E[\lambda_i/\sigma_i|data]
    y_var = mean(est_draw.het_draw(ix_out,:,1)./sqrt(est_draw.het_draw(ix_out,:,2)))';
    
    ix_group = cell(n_group,1); % order: [Low, high]
    ix_group{1} = y_var<-2*(i_chargeoff==6)-1*(i_chargeoff==7); % cutoff: -2 for RRE, -1 for CC
    ix_group{2} = ~ix_group{1};
    
    yylim = [min(y_var) max(y_var)];
    
    %% x variable = bank characteristics
    % Read raw data
    raw = importdata([data_path 't0_' num2str(yr) 'Q' num2str(qtr) '_' ...
        chargeoff_names{i_chargeoff} '_bankvar.txt']);
    x_alls = [log(raw(:,2)) raw(:,i_chargeoff+2)./sum(raw(:,3:10),2) raw(:,11:end)];
    
    %% Loop over bank characteristics
    for i_bankvar = 1%:n_bankvar
        x_var = x_alls(:,i_bankvar);
        xxlim = [min(x_var) max(x_var)];
        
        % Only plot x variables observed for some units and with nontrival variation
        if ~all(isnan(x_var)) && nanvar(x_var)>1e-10 
            %% Plot graph
            F1 = figure;
            
            %% Scatter plot with LAD line for each group
            hold on           
            for i_group = 1:n_group
                scatter(x_var(ix_group{i_group}),y_var(ix_group{i_group}),25,color_group(i_group,:));
                
                coefs = qr_standard(x_var(ix_group{i_group}),y_var(ix_group{i_group}),.5, ...
                    'test','wald','method','interior');
                h = refline(coefs([2 1]));
                h.Color = color_group(i_group,:)*.7; % slightly darker than the scatter
                h.LineStyle = '-';
                h.LineWidth = 2;
            end
            hold off
            
            xlim(xxlim)
            ylim(yylim+diff(yylim)*[-1/3-0.05 0.05])
            xlabel(bankvar_print{i_bankvar})
            
            %% Logit grouping will all units
            coefs = glmfit(x_var,ix_group{2},'binomial');
            x_grid = linspace(xxlim(1),xxlim(2),100);
            y_grid = glmval(coefs,x_grid,'logit');

            yyaxis right
            jbfill(xxlim,ones(1,2),zeros(1,2),[.7 .7 .7],[.7 .7 .7],1,0.5);
            hold on
            plot(x_grid,y_grid,'k-','linewidth',2);
            hold off
            
            xlim(xxlim)
            ylim([0 4])
            set(gca,'YColor','k')
            yticks([0 1])
            
            %% Save graph
            graph_tag = [result_path spec_smpl '_' bankvar_names{i_bankvar}];
            graph_out(F1,graph_tag,[],SFont,graph_size,lgd);
        end
    end
end

%% Close files
close all