# Generate graphs: Charge-Off Predictive Tail Probabilities, Spatial Dimension
# (for selected samples: v_smpl0 = RRE and CC samples starting at 2007Q2
#                        v_smpl1 = RRE/CC sample starting at 2015Q2/2012Q3)

# Load packages
library(R.matlab)
library(maps)
library(mapproj) # mapproj is used for projection="polyconic"
library(pals) # pasl is used for colormaps

# Set working directory and output paths
# https://stackoverflow.com/a/15373917
thisFile = function() {
  cmdArgs = commandArgs(trailingOnly = FALSE)
  needle = "--file="
  match = grep(needle, cmdArgs)
  if (length(match) > 0) { # Rscript
    return(normalizePath(sub(needle, "", cmdArgs[match])))
  } else { # 'source'd via R console
    return(normalizePath(sys.frames()[[1]]$ofile))
  }
}

setwd(dirname(dirname(dirname(thisFile()))))
# If error occurred in setting the working directory automatically, please set it manually to 
# the "ReplicationFiles" root directory by setwd().

data.dir = "application/data/"
result.dir = "application/results/"

# Load dataset
yr0 = 1994
qtr0 = 1
yr1 = 2018
qtr1 = 4
T_all = (yr1-yr0)*4+(qtr1-qtr0+1)
tag0 = paste0('_',as.character(yr0),'Q',as.character(qtr0),'_',
              as.character(yr1),'Q',as.character(qtr1))

load(file=paste0(data.dir,"bank",tag0,".RData"))

# ==================================================================================================
# Prepare
chargeoff_names = c("ci", "cld", "mf", "cre", "hlc", "rre", "cc", "con")
het_names = c("tailprob")
n_het = length(het_names)

mat_smpl = matrix(c(2007, 2, 6, 2007, 2, 7, 2015, 2, 6, 2012, 3, 7), nrow = 3)
n_smpl = ncol(mat_smpl)
if (n_smpl == 1) {
  is_new_time = TRUE
} else if (n_smpl == 2) {
  is_new_time = c(TRUE, sum(abs(mat_smpl[1:2, -1]-mat_smpl[1:2, -n_smpl]))>0)
} else {
  is_new_time = c(TRUE, colSums(abs(mat_smpl[1:2, -1]-mat_smpl[1:2, -n_smpl]))>0)
}
ix_new_chargeoff = match(unique(mat_smpl[3, ]), mat_smpl[3, ])

# ==================================================================================================
# Load results
het_loc = list()

for (i_smpl in 1:n_smpl) {
  yr = mat_smpl[1, i_smpl]
  qtr = mat_smpl[2, i_smpl]
  i_chargeoff = mat_smpl[3, i_smpl]
  
  # Bank location data
  if (is_new_time[i_smpl]) {
    t0_yq = yr+(qtr-1)/4
    loc = bank[(bank$y.q>=t0_yq-0.01)&(bank$y.q<=t0_yq+0.01), c("RSSDID", "STCNTY")]
  }
  
  # Estimated heterogeneous results, from MATLAB output
  het_mat = as.data.frame(readMat(paste0(data.dir, "t0_", as.character(yr), "Q", as.character(qtr), 
                                         "_", chargeoff_names[i_chargeoff], "_het.mat"))[1])
  colnames(het_mat) = c("RSSDID", het_names)
  
  het_loc[[i_smpl]] = merge(het_mat, loc, by = "RSSDID")
}

# ==================================================================================================
# Group banks by quantile
n_group = 6

# Cutoff points: same within each loan category
v_quantile = seq(from = 0, to = 1, length.out = n_group+1)
v_chargeoff = unique(mat_smpl[3, ])
n_chargeoff = length(v_chargeoff)
q_group = array(dim = c(n_group+1, n_het, n_chargeoff), 
                dimnames = list(v_quantile, het_names, chargeoff_names[v_chargeoff]))

for (i_chargeoff in 1:n_chargeoff) {
  ix_smpl = which(mat_smpl[3, ]==v_chargeoff[i_chargeoff])
  aux = het_loc[[ix_smpl[1]]]
  
  for (i_smpl in ix_smpl[2:length(ix_smpl)]) {
    aux = rbind(aux, het_loc[[i_smpl]])
  }
  
  for (i_het in 1:n_het) {
    q_group[, i_het, i_chargeoff] = quantile(aux[, i_het+1], v_quantile)
  }
}

# Color map: shades of blue
n_color_aux = floor(n_group*1.2) 
color_aux = brewer.blues(n_color_aux)
color_group = c(color_aux[n_color_aux-((n_group-1):0)], "#E6E6E6")
# first entry: start w a shade a bit darker than equal spacing
# second entry: grey for unmatched county

# Legends: keep two digits
aux_q_2digit = 10^(floor(log10(abs(q_group)))-1)
q_group_lgd = round(q_group/aux_q_2digit)*aux_q_2digit

# ==================================================================================================
# County-level Plots
all_county_fips = county.fips$fips[match(map("county", plot = FALSE)$names, county.fips$polyname)]

for (i_smpl in 1:n_smpl) {
  yr = mat_smpl[1, i_smpl]
  qtr = mat_smpl[2, i_smpl]
  i_chargeoff = mat_smpl[3, i_smpl]
  data_county = het_loc[[i_smpl]][, c(het_names, "STCNTY")]
  
  # Aggregate by state-county FIPS
  agg_county = aggregate(.~STCNTY, data_county, FUN = mean, na.rm = TRUE, na.action = na.pass)
  
  # Loop over heterogeneous features
  for (i_het in n_het) {
    # Aggregate by state-county FIPS
    the_agg_county = agg_county[, c(1, i_het+1)]
    colnames(the_agg_county) = c("county_fips", "county_mean")
    
    # Define color buckets
    the_q_group = q_group[, i_het, which(v_chargeoff==i_chargeoff)]
    the_agg_county$color_bucket = as.numeric(cut(the_agg_county$county_mean, the_q_group))
    
    # Align data with map polygons
    color_matched = the_agg_county$color_bucket[match(all_county_fips, the_agg_county$county_fips)]
    color_matched[is.na(color_matched)] = n_group+1 # grey for unmatched county
    
    # Plot map
    png(paste0(result.dir, "app_", as.character(yr), "Q", as.character(qtr), "_", 
               chargeoff_names[i_chargeoff], "_", het_names[i_het], "_county.png"), 
        width = 6, height = 4, unit = "in", res = 108)
    par(mar = c(.1, .1, 0.1, 0.1))
    map("county", col = color_group[color_matched], fill = TRUE, resolution = 0, lty = 0, 
        projection = "polyconic")
    map("state", col = "white", fill = FALSE, add = TRUE, lty = 1, lwd = 0.2, 
        projection = "polyconic") # white state borders
    dev.off()
    
    # Generate legend for each loan category
    if (i_smpl %in% ix_new_chargeoff) {
      the_q_group_lgd = q_group_lgd[,i_het,which(v_chargeoff==i_chargeoff)]
      leg.txt = c(paste("<", the_q_group_lgd[2]), 
                  paste0("[", the_q_group_lgd[2:(n_group-1)], ",", the_q_group_lgd[3:n_group], ")"), 
                  paste(">=", the_q_group_lgd[n_group]))
      
      png(paste0(result.dir, "app_", chargeoff_names[i_chargeoff], "_", het_names[i_het], 
                 "_lgd.png"),
          width = 18, height = 3, unit = "in", res = 108)
      plot(NULL, xaxt = "n", yaxt = "n", bty = "n", ylab = "", xlab = "", xlim = 0:1, ylim = 0:1)
      legend("center", leg.txt, fill = color_group, ncol = 6, bty = "n", cex = 1.5, pt.cex = 1)
      dev.off()
    }
    
  }
}