% Generate graphs: Posterior Predictive Checks: Cross-sectional Distribution of Sample Statistics
% (for selected samples: v_smpl0 = RRE and CC samples starting at 2007Q2)

%% Setup style
graph_size = [6 4];
lgd = [];
SFont = 16;

n_vec = 100;

stat_names = {'ac_pos','after0','before0','ac_MM_all'};
n_stat = length(stat_names);
xxlim_vec = nan(n_stat,2);
qxlim_vec = [0 1; 0 .975; 0 .975; .05 .95]; % quantiles for xlim, n_stat x 2
x_vec = nan(n_stat,n_vec);
f_vec = nan(n_stat,n_vec);

thin = 10; % use every 10th MCMC draw, change if needed

i_est = 1; % Flexible CRE, heterosk.
is_load_est = 1; % load estimation results

%% Loop over samples
for i_smpl0 = 1:n_smpl0
    %% Prepare: sample i and forecast
    sub_smpl_i; % subroutine: prepare sample i
    sub_fcst_prep; % subroutine: "Mean" and "StdD" of the trunc normal pred dist for each MCMC draw
    
    X0 = [ones(N,1) permute(X(2,:,:),[2,3,1])];
    
    %% 1. Density y_{i,T+1} | (y_{i,T+1}>0): actual data  
    xxlim0 = quantile(Y(T+2,:),[.025 .975]);
    x_vec0 = linspace(xxlim0(1),xxlim0(2),n_vec);
    [f_vec0,~] = ksdensity(Y(T+2,Y(T+2,:)~=0),x_vec0);
    P0 = mean(Y(T+2,:)==0);
    
    %% 2. Dist of frequency of zero charge-off rates: actual data
    hist0 = hist(sum(Y==0),0:T+2)/N;
    hist_est = nan(N_draw/thin,T+3);
    
    %% 3. Time series statistics: actual data
    Y_aux = Y;
    for i_stat = 1:n_stat
        stat_aux = nan(1,N);
        for i = 1:N
            Y_aux1 = Y_aux(1:end-1,i);
            Y_aux2 = Y_aux(2:end,i);
            if i_stat == 1 % correlation of y_it and y_{i,t-1} if both are positive
                ix = Y_aux(:,i)>0;
                ix = logical(ix(1:end-1).*ix(2:end));
                if sum(ix) >= 2
                    aux = corrcoef(Y_aux1(ix),Y_aux2(ix));
                    stat_aux(i) = aux(1,2);
                end
            elseif i_stat == 2 % mean of y_it after obs 0
                ix = diff(Y_aux(:,i)>0)==1;
                if ~isempty(ix)
                    stat_aux(i) = mean(Y_aux2(ix));
                end
            elseif i_stat == 3 % mean of y_it before obs 0
                ix = diff(Y_aux(:,i)>0)==-1;
                if ~isempty(ix)
                    stat_aux(i) = mean(Y_aux1(ix));
                end
            else % robust correlation of y_it and y_{i,t-1} for all obs
                ix = true(1,length(Y_aux1));
                if sum(ix) >= 2 && var(Y_aux1(ix))*var(Y_aux2(ix))~=0 
                    results = MMreg(Y_aux2(ix),Y_aux1(ix),'msg',0);
                    stat_aux(i) = results.beta(2);                  
                end
            end
        end
        xxlim_vec(i_stat,:) = quantile(stat_aux,qxlim_vec(i_stat,:));
        x_vec(i_stat,:) = linspace(xxlim_vec(i_stat,1),xxlim_vec(i_stat,2),n_vec);
        [f_vec(i_stat,:),~] = ksdensity(stat_aux,x_vec(i_stat,:));
    end
    
    %% Simulate data based on posterior draws  
    % Have to load these two sets of MCMC draws b/c "the ranges for MatFile objects must increase 
    % in equally spaced intervals" whereas the ix_aux below is not.
    mm_lnsigma2_draw = est_draw.mm_lnsigma2_draw;
    vv_lnsigma2_draw = est_draw.vv_lnsigma2_draw;
    
    Y1_predcheck = nan(N_draw/thin,N);
    stat_simul = nan(N_draw/thin,N,n_stat);
    
    %% Loop over posterior draws
    for ii_draw = 1:N_draw/thin
        i_draw = (ii_draw-1)*thin+1+B_draw;
        
        %% Simulate data
        rrho = est_draw.param_draw(i_draw,1:3)';
        
        % (lambda_i, y*_i0)
        Y_star_aux = nan(T+2,N);
        
        pp_aux = cumsum(est_draw.pp_draw(i_draw,:,1))';
        uu_aux = rand(1,N);
        ix_aux = sum(uu_aux>pp_aux)+1;
        
        mm_aux = squeeze(est_draw.mm_lambday0_draw(i_draw,:,:,:));
        vv_aux = squeeze(est_draw.vv_lambday0_draw(i_draw,:,:,:));
        
        mm_aux2 = nan(N,2);
        for i = 1:N
            mm_aux2(i,:) = X0(i,:)*mm_aux(:,:,ix_aux(i));
        end
        
        aux = mvnrnd(mm_aux2,vv_aux(:,:,ix_aux))';
        
        llambda = aux(1,:);
        Y_star_aux(1,:) = aux(2,:);
        
        % sigma2_i
        pp_aux = cumsum(est_draw.pp_draw(i_draw,:,2))';
        uu_aux = rand(1,N);
        ix_aux = sum(uu_aux>pp_aux)+1;
        ssigma2 = exp(randn(1,N).*sqrt(vv_lnsigma2_draw(i_draw,ix_aux)) ...
            +mm_lnsigma2_draw(i_draw,ix_aux));
        
        % y*_it
        for t = 2:T+2
            Y_star_aux(t,:) = ([Y_star_aux(t-1,:)' squeeze(X(t,:,:))]*rrho)' ...
                +llambda+sqrt(ssigma2).*randn(1,N);
        end
        
        % y_it
        Y_aux = max(Y_star_aux,0);
        
        %% 1. y_{i,T+1}: simulated data
        Y1_predcheck(ii_draw,:) = Y_aux(end,:);
        
        %% 2. Dist of frequency of zero charge-off rates: simulated data
        hist_est(ii_draw,:) = hist(sum(Y_aux==0),0:T+2)/N;
        
        %% 3. Time series statistics: simulated data
        for i_stat = 1:n_stat
            stat_aux = nan(1,N);
            for i = 1:N
                Y_aux1 = Y_aux(1:end-1,i);
                Y_aux2 = Y_aux(2:end,i);
                if i_stat == 1 % correlation of y_it and y_{i,t-1} if both are positive
                    ix = Y_aux(:,i)>0;
                    ix = logical(ix(1:end-1).*ix(2:end));
                    if sum(ix) >= 2
                        aux = corrcoef(Y_aux1(ix),Y_aux2(ix));
                        stat_aux(i) = aux(1,2);
                    end
                elseif i_stat == 2 % mean of y_it after obs 0
                    ix = diff(Y_aux(:,i)>0)==1;
                    if ~isempty(ix)
                        stat_aux(i) = mean(Y_aux2(ix));
                    end
                elseif i_stat == 3 % mean of y_it before obs 0
                    ix = diff(Y_aux(:,i)>0)==-1;
                    if ~isempty(ix)
                        stat_aux(i) = mean(Y_aux1(ix));
                    end
                else % robust correlation of y_it and y_{i,t-1} for all obs
                    ix = true(1,length(Y_aux1));
                    if sum(ix) >= 2 && var(Y_aux1(ix))*var(Y_aux2(ix))~=0
                        results = MMreg(Y_aux2(ix),Y_aux1(ix),'msg',0);
                        stat_aux(i) = results.beta(2);
                    end
                end
            end
            stat_simul(ii_draw,:,i_stat) = stat_aux;
        end
    end
    
    %% 1. Density y_{i,T+1} | (y_{i,T+1}>0): graph
    F1 = figure;
    hold on
    for ii_draw = 1:N_draw/thin
        [f_vec_aux,~] = ksdensity(Y1_predcheck(ii_draw,Y1_predcheck(ii_draw,:)~=0),x_vec0);
        patchline(x_vec0',f_vec_aux','linestyle','-','edgecolor','b','linewidth',2, ...
            'edgealpha',0.05);
    end
    plot(x_vec0,f_vec0,'k-','linewidth',2)
    hold off
    xlim(xxlim0);
    P0_aux = mean(Y1_predcheck==0,2);
    q_P0 = quantile(P0_aux,[.025,.975]);
    % title(['P0 = ' num2str(P0,2) ' (true), ' num2str(mean(P0_aux),2) ' [' num2str(q_P0(1),2) ',' ...
    %     num2str(q_P0(2),2) '] (est)'])
    
    % Save graph
    graph_tag = [result_path spec_smpl '_check_cs_YiT1'];
    graph_out(F1,graph_tag,[],SFont,graph_size,lgd);
    
    %% 2. Dist of frequency of zero charge-off rates: graph
    F1 = figure;
    bar(0:T+2,hist0,'FaceColor',[.7 .7 .7],'LineStyle','none')
    hold on
    for ii_draw = 1:N_draw/thin
        patchline((0:T+2)',hist_est(ii_draw,:)','linestyle','-','edgecolor','b','linewidth',2, ...
            'edgealpha',0.05);
    end
    hold off
    
    % Save graph
    graph_tag = [result_path spec_smpl '_check_cs_count0s'];
    graph_out(F1,graph_tag,[],SFont,graph_size,lgd);
    
    %% 3. Time series statistics: graph
    for i_stat = 1:n_stat
        F1 = figure;
        hold on
        for ii_draw = 1:N_draw/thin
            [f_vec_aux,~] = ksdensity(stat_simul(ii_draw,:,i_stat),x_vec(i_stat,:));
            patchline(x_vec(i_stat,:)',f_vec_aux','linestyle','-','edgecolor','b','linewidth',2, ...
                'edgealpha',0.05);
        end
        plot(x_vec(i_stat,:),f_vec(i_stat,:),'k-','linewidth',2)
        hold off
        xlim(xxlim_vec(i_stat,:));
        
        % Save graph
        graph_tag = [result_path spec_smpl '_check_' stat_names{i_stat}];
        graph_out(F1,graph_tag,[],SFont,graph_size,lgd);
    end
end

%% Close files and clean up space
close all
clear mm_lnsigma2_draw vv_lnsigma2_draw;