% Generate tables: 
% - Summary Statistics for Bank Characteristics
% - Regression of heterogeneous coefficients on bank characteristics
% (for selected samples: v_smpl0 = RRE and CC samples starting at 2007Q2)

%% Setup style
bankvar_print = ... % row labels in output tables
   {'Log Assets      ';
    'Loan Fraction   ';
    'Capital-Asset   ';
    'Loan-Asset      ';
    'ALLL-Loan       ';
    'Diversification ';
    'Ret.\ on Assets ';
    'OCA             '};
n_bankvar = length(bankvar_print);

file_names = {'bankvar_stat','bankvar_reg'};
n_file = length(file_names);
file_id = nan(1,n_file);
for i_file = 1:n_file
    file_id(i_file) = fopen([result_path spec '_' file_names{i_file} '.txt'],'w'); 
    % change to 'w'/'a' if needed
end
n_group = 2;
n_stat = 2;
ncol_file = n_stat*(n_group+1)*ones(n_file,1);
nrow_file = n_bankvar+[1 2]';

add_print = cell(2,n_file); % additional rows in output tables
add_print{1,1} = 'Sample Size     ';
add_print(:,2) = ...
   {'Intercept       ';
    'Pseudo R^2      '};

i_est = 1; % Flexible CRE, heterosk.
is_load_est = 1; % load estimation results

%% Loop over samples
for i_smpl0 = 1:n_smpl0
    %% Prepare: sample i
    sub_smpl_i; % subroutine: prepare sample i
    
    %% y variable = E[\lambda_i/\sigma_i|data]
    y_var = mean(est_draw.het_draw(ix_out,:,1)./sqrt(est_draw.het_draw(ix_out,:,2)))';
    
    ix_group = cell(n_group,1); % order: [Low, high]
    ix_group{1} = y_var<-2*(i_chargeoff==6)-1*(i_chargeoff==7); % cutoff: -2 for RRE, -1 for CC
    ix_group{2} = ~ix_group{1};
    
    %% x variable = bank characteristics
    % Read raw data
    raw = importdata([data_path 't0_' num2str(yr) 'Q' num2str(qtr) '_' ...
        chargeoff_names{i_chargeoff} '_bankvar.txt']);    
    x_all = [log(raw(:,2)) raw(:,i_chargeoff+2)./sum(raw(:,3:10),2) raw(:,11:end)];

    % Select x variables observed for some units and with nontrival variation
    v_x_all = nanvar(x_all);
    ix_reg = find(~isnan(v_x_all) & v_x_all>1e-10);    
    x_var = x_all(:,ix_reg);
    ix_reg1 = [n_bankvar+1 ix_reg]; % index with intercept as the last entry
    
    % Indices for units with a full set of selected x variables, i.e., regression sample
    ix_finite = all(isfinite(x_var),2);
    
    %% Prepare: tables
    table_stat = nan(nrow_file(1),n_stat,n_group+1); 
        % 1st dim order: [Bank characteristics, Sample Size]
        % 2nd dim order: [Mean, StdD]
        % 3rd dim order: [Low, High, All]
    
    table_reg = nan(nrow_file(2),n_stat,n_group+1);
        % 1st dim order: [Bank characteristics, Intercetpt, Pseudo R2]
        % 2nd dim order: [Coef, StdD]
        % 3rd dim order: [Low, High, Logit]
    
    %% Logit grouping will all units
    xx_var = x_var(ix_finite,:);
    yy_var = ix_group{2}(ix_finite);
    nn = length(yy_var);
    
    % Summary statistics for all units
    table_stat([ix_reg end],:,3) = [mean(xx_var)' std(xx_var)'; nn nan(1)];
    
    % Logit coefs and StdD
    [coefs,~,stats] = glmfit(xx_var,yy_var,'binomial');
    table_reg(ix_reg1,1,3) = coefs;
    table_reg(ix_reg1,2,3) = stats.se;
    
    % Pseudo R2 (McFadden, 1973)
    % loglh(all)
    mus = glmval(coefs,xx_var,'logit');
    l_all = sum(yy_var.*log(mus)+(1-yy_var).*log(1-mus));
    
    % loglh(intcpt)
    xx_var = ones(nn,1);
    coefs = glmfit(xx_var,yy_var,'binomial','Constant','off');
    mus = glmval(coefs,xx_var,'logit','Constant','off');
    l_intcpt = sum(yy_var.*log(mus)+(1-yy_var).*log(1-mus));
    
    % Pseudo R2
    table_reg(end,1,3) = 1-l_all/l_intcpt; % Note that log likelihood is always negative for logit.
    
    %% LAD regression for each group 
    for i_group = 1:n_group
        ix_group_finite = ix_group{i_group} & ix_finite;
        
        xx_var = x_var(ix_group_finite,:);
        yy_var = y_var(ix_group_finite);
        nn = length(yy_var);
        
        % Summary statistics for each group
        table_stat([ix_reg end],:,i_group) = [mean(xx_var)' std(xx_var)'; 
                                              nn nan(1)];
        
        % LAD coefs and StdD
        [table_reg(ix_reg1,1,i_group),table_reg(ix_reg1,2,i_group)] = ...
            qr_standard(xx_var,yy_var,.5,'test','wald','method','interior');
        
        % Pseudo R2 (Koenker and Machado, 1999)
        obj_all = sum(abs(yy_var-[ones(nn,1) xx_var]*table_reg(ix_reg1,1,i_group)));
        obj_intcpt = sum(abs(yy_var-median(yy_var)));
        table_reg(end,1,i_group) = 1-obj_all/obj_intcpt;
    end
    
    %% Generate output tables
    table_out = {reshape(table_stat,[nrow_file(1),n_stat*(n_group+1)]), ...
        reshape(table_reg,[nrow_file(2),n_stat*(n_group+1)])}; 
        % stat order: [Mean, StdD] x [Low, High, All] 
        % reg order:  [Coef, StdD] x [Low, High, Logit]

    for i_file = 1:n_file
        combined_print = [bankvar_print; add_print(:,i_file)];
        fprintf(file_id(i_file), ...
            [num2str(yr) 'Q' num2str(qtr) ' ' chargeoff_names{i_chargeoff} '\r\n']);  
        
        for i_row = 1:nrow_file(i_file)
            if i_file == 1 && i_row == nrow_file(i_file) % sample size
                fprintf(file_id(i_file), ...
                    ['%4.20s ' repmat('& %7.0f ',1,ncol_file(i_file)) ' %4.6s\r\n'], ...
                    combined_print{i_row},table_out{i_file}(i_row,:),'\\ ');
            else % other entries
                fprintf(file_id(i_file), ...
                    ['%4.20s ' repmat('& %7.3f ',1,ncol_file(i_file)) ' %4.6s\r\n'], ...
                    combined_print{i_row},table_out{i_file}(i_row,:),'\\ ');
            end
        end   
        
        fprintf(file_id(i_file),'\r\n');
    end
end

%% Close files
for i_file = 1:n_file
    fclose(file_id(i_file));
end