%% Forecasting with a Panel Tobit Model
%  Laura Liu, Hyungsik Roger Moon, and Frank Schorfheide

%  Monte Carlo Simulation
%  Estimate the model using different methods 

clear all
close all
clc

addpath('../tools');
addpath('estimators');
mkdir temp

temp_path = 'temp/';
i_dgp = 1; %1: 45% zeros
           %2: 60% zeros
           %3: 75% zeros
spec = ['dgp' num2str(i_dgp)];

n_core = feature('numcores'); % number of cores: change if needed

%% General DGP parameters
N = 1000;
T = 10;

rrho = 0.8;
mean_ssigma2 = 1;

all_lambda_shift = [.25 -.15 -.7]; % shifts in lambda distributin for all DGPs
m_llambda = [2 -1/4]+all_lambda_shift(i_dgp);
v_llambda = [1/2 1/2];
p_llambda = [1/9 8/9];
ssigma2_normalization = p_llambda(1)*exp(m_llambda(1)+v_llambda(1)/2) ...
    +p_llambda(2)*exp(m_llambda(2)+v_llambda(2)/2);

n_sim = 100;
n_param = 3;

%% Estimation setup
est_names = {'flex_heterosk','normal_heterosk','flex_homosk','normal_homosk','pool_tobit', ...
    'pool_linear'};
n_est = length(est_names);

B_draw = 1000; % total number of MCMC draws = 10,000; burn-in = 1000
N_draw = 9000; 

ttau = 5; % tuning parameter in priors

n_sfcst = 7; % order: [Cover. Freq., Ave. Len., Empty Set, {0}, [0,b], {0}U[0,b], Other]
n_dfcst = 2; % order: [LPS, CRPS]

%% Initialize output arrays
Y_out = cell(1,n_sim);
sfcstPtwise_stat = cell(n_est,n_sim);
sfcstAvg_stat = cell(n_est,n_sim);
dfcst_stat = cell(n_est,n_sim);
param_est = cell(n_est,n_sim);
het_est = cell(n_est,n_sim);

%% Loop over repetitions
delete(gcp('nocreate'))
parpool('local',n_core);

parfor i_sim = 1:n_sim
    %% DGP  
    rng(i_sim,'twister');
    llambda = [randn(1,round(N*p_llambda(1)))*sqrt(v_llambda(1))+m_llambda(1) ...
        randn(1,round(N*p_llambda(2)))*sqrt(v_llambda(2))+m_llambda(2)];
    ssigma2 = exp([randn(1,round(N*p_llambda(1)))*sqrt(v_llambda(1))+m_llambda(1) ...
        randn(1,round(N*p_llambda(2)))*sqrt(v_llambda(2))+m_llambda(2)]) ...
        /ssigma2_normalization*mean_ssigma2;
    ssigma2 = ssigma2(randperm(length(ssigma2)));
    
    Y_star = nan(T+2,N);
    u = randn(T+1,N);
    Y_star(1,:) = randn(1,N);
    for t = 2:T+2
        Y_star(t,:) = rrho*Y_star(t-1,:)+llambda+sqrt(ssigma2).*u(t-1,:);
    end
    Y = max(Y_star,0);
    Y_out{i_sim} = Y;
    
    %% Initialize output arrays inside parfor loop (needed due to requirement of parfor)    
    sfcstPtwise_aux = cell(n_est,1);
    sfcstAvg_aux = cell(n_est,1);
    dfcst_aux = cell(n_est,1);
    param_aux = cell(n_est,1);
    het_aux = cell(n_est,1);
    
    %% Tobit estimators    
    warning('error','MATLAB:nearlySingularMatrix');
    warning('error','MATLAB:illConditionedMatrix');
    
    for i_est = 1:n_est        
        disp(['i_sim = ' num2str(i_sim) ', i_est = ' num2str(i_est)])
        
        param0 = cell(2,1);
        if ismember(i_est,[1 3])  % Flexible RE
            param0{1} = 'flexible';
        elseif ismember(i_est,[2 4])  % Normal RE
            param0{1} = 'normal';
        end
        param0{2} = ttau;
        
        save_file = cell(1,2);
        if i_sim == 1 && ((i_dgp==1 && i_est<=2) || (i_dgp>1 && i_est==1))
            save_file{1} = [temp_path spec '_']; % save MCMC details with provided path and spec
        else
            save_file{1} = []; % not save MCMC details
        end
        save_file{2} = ['isim' num2str(i_sim) '_' est_names{i_est}];
            % repetition and estimator, for (1) filename and (2) displaying MCMC progress
        
        is_err = 1;
        count = 1;
        while is_err == 1 
            % Normal Heterosk. estimator encounters singularity issue in rare cases.                          
            % Restart the MCMC would resolve the issue.
            if count > 1
                disp(['count = ' num2str(count)]);
            end
            
            try              
                if i_est <= 2  % Heteroskedastic RE     
                    [sfcstPtwise_aux{i_est},sfcstAvg_aux{i_est},dfcst_aux{i_est},het_aux{i_est}, ...
                        param_aux{i_est}] = tobit_heterosk_re(Y,[B_draw N_draw],param0,save_file);                  
                elseif i_est <= 4  % Homoskedastic RE                 
                    [sfcstPtwise_aux{i_est},sfcstAvg_aux{i_est},dfcst_aux{i_est},het_aux{i_est}, ...
                        param_aux{i_est}] = tobit_re(Y,[B_draw N_draw],param0,save_file);                    
                elseif i_est == 5 % Pooled Tobit                  
                    [sfcstPtwise_aux{i_est},sfcstAvg_aux{i_est},dfcst_aux{i_est}, ...
                        param_aux{i_est}] = pool_tobit(Y,[B_draw N_draw],param0,save_file);                  
                else % Pooled Linear                
                    [sfcstPtwise_aux{i_est},sfcstAvg_aux{i_est},dfcst_aux{i_est}, ...
                        param_aux{i_est}] = pool_linear(Y,[B_draw N_draw],param0,save_file);                    
                end  
                
                is_err = 0; 
                
            catch ME 
                disp(ME.message)
                count = count+1;              
            end           
        end
    end
    
    %% Output    
    sfcstPtwise_stat(:,i_sim) = sfcstPtwise_aux;
    sfcstAvg_stat(:,i_sim) = sfcstAvg_aux;
    dfcst_stat(:,i_sim) = dfcst_aux;
    param_est(:,i_sim) = param_aux;
    het_est(:,i_sim) = het_aux;    
end

%% Save
delete(gcp('nocreate'))
save([temp_path spec '.mat'])

rmpath('../tools');
rmpath('estimators');
